<?php

/**
 * @file
 * Editor CKEditor hooks implemented by the Editor CKEditor module.
 */

/**
 * Implements hook_editor_ckeditor_plugins().
 */
function editor_ckeditor_editor_ckeditor_plugins() {
  $image_prefix = drupal_get_path('module', 'editor_ckeditor') . '/images/buttons/';

  $buttons = array(
    // "basicstyles" plugin.
    'Bold' => array(
      'label' => t('Bold'),
      'required_html' => array(array('tags' => array('strong'))),
    ),
    'Italic' => array(
      'label' => t('Italic'),
      'required_html' => array(array('tags' => array('em'))),
    ),
    'Underline' => array(
      'label' => t('Underline'),
      // A class is used on spans for underline.
      'required_html' => array(array('tags' => array('span'))),
    ),
    'Strike' => array(
      'label' => t('Strike-through'),
      'required_html' => array(array('tags' => array('del'))),
    ),
    'Superscript' => array(
      'label' => t('Superscript'),
      'required_html' => array(array('tags' => array('sup'))),
    ),
    'Subscript' => array(
      'label' => t('Subscript'),
      'required_html' => array(array('tags' => array('sub'))),
    ),
    // "removeformat" plugin.
    'RemoveFormat' => array(
      'label' => t('Remove format'),
    ),
    // "justify" plugin.
    'JustifyLeft' => array(
      'label' => t('Align left'),
      'required_html' => array(array('tags' => array('p'), 'classes' => array('text-align-left'))),
    ),
    'JustifyCenter' => array(
      'label' => t('Align center'),
      'required_html' => array(array('tags' => array('p'), 'classes' => array('text-align-center'))),
    ),
    'JustifyRight' => array(
      'label' => t('Align right'),
      'required_html' => array(array('tags' => array('p'), 'classes' => array('text-align-right'))),
    ),
    'JustifyBlock' => array(
      'label' => t('Justify'),
      'required_html' => array(array('tags' => array('p'), 'classes' => array('text-align-justify'))),
    ),
    // "list" plugin.
    'BulletedList' => array(
      'label' => t('Bullet list'),
      'image_rtl' => $image_prefix . '/bulletedlist-rtl.png',
      'required_html' => array(array('tags' => array('ul', 'li'))),
    ),
    'NumberedList' => array(
      'label' => t('Numbered list'),
      'image_rtl' => $image_prefix . '/numberedlist-rtl.png',
      'required_html' => array(array('tags' => array('ol', 'li'))),
    ),
    // "indent" plugin.
    'Outdent' => array(
      'label' => t('Outdent'),
      'image_rtl' => $image_prefix . '/outdent-rtl.png',
      'required_html' => array(array('tags' => array('ul', 'ol'))),
    ),
    'Indent' => array(
      'label' => t('Indent'),
      'image_rtl' => $image_prefix . '/indent-rtl.png',
      'required_html' => array(array('tags' => array('ul', 'ol'))),
    ),
    // "undo" plugin.
    'Undo' => array(
      'label' => t('Undo'),
      'image_rtl' => $image_prefix . '/undo-rtl.png',
    ),
    'Redo' => array(
      'label' => t('Redo'),
      'image_rtl' => $image_prefix . '/redo-rtl.png',
    ),
    // "blockquote" plugin.
    'Blockquote' => array(
      'label' => t('Blockquote'),
      'required_html' => array(array('tags' => array('blockquote'))),
    ),
    // "horizontalrule" plugin
    'HorizontalRule' => array(
      'label' => t('Horizontal rule'),
      'required_html' => array(array('tags' => array('hr'))),
    ),
    // "clipboard" plugin.
    'Cut' => array(
      'label' => t('Cut'),
    ),
    'Copy' => array(
      'label' => t('Copy'),
    ),
    'Paste' => array(
      'label' => t('Paste'),
    ),
    // "pastetext" plugin.
    'PasteText' => array(
      'label' => t('Paste Text'),
      'image_rtl' => $image_prefix . '/pastetext-rtl.png',
    ),
    // "pastefromword" plugin.
    'PasteFromWord' => array(
      'label' => t('Paste from Word'),
      'image_rtl' => $image_prefix . '/pastefromword-rtl.png',
    ),
    // "specialchar" plugin.
    'SpecialChar' => array(
      'label' => t('Character map'),
    ),
    'Format' => array(
      'label' => t('HTML block format'),
      'image_alternative' => '<span class="ckeditor-button-dropdown">' . t('Format') . '<span class="ckeditor-button-arrow"></span></span>',
    ),
    // "table" plugin.
    'Table' => array(
      'label' => t('Table'),
      'required_html' => array(array('tags' => array('table', 'thead', 'tbody', 'tr', 'td', 'th'))),
    ),
    // "showblocks" plugin.
    'ShowBlocks' => array(
      'label' => t('Show blocks'),
      'image_rtl' => $image_prefix . '/showblocks-rtl.png',
    ),
    // "sourcearea" plugin.
    'Source' => array(
      'label' => t('Source code'),
    ),
    // "maximize" plugin.
    'Maximize' => array(
      'label' => t('Maximize'),
    ),
    // No plugin, separator "button" for toolbar builder UI use only.
    '-' => array(
      'label' => t('Separator'),
      'image_alternative' => '<span class="ckeditor-separator" title="' . t('Button separator') . '" aria-label="' . t('Button separator') . '">&nbsp;</span>',
      'attributes' => array('class' => array('ckeditor-button-separator')),
      'multiple' => TRUE,
    ),
  );

  // Populate image locations that match button names.
  foreach ($buttons as $button_name => &$button) {
    if (!isset($button['image_alternative']) && !isset($button['image'])) {
      // Because button names are ASCII text, drupal_strtolower() is not needed.
      $button['image'] = $image_prefix . strtolower($button_name) . '.png';
    }
  }

  // List all the basic plugin buttons as an "internal" plugin.
  $plugins['internal'] = array(
    'label' => t('CKEditor core'),
    'internal' => TRUE,
    'contextual' => TRUE,
    'buttons' => $buttons,
    'config callback' => 'editor_ckeditor_internal_config',
  );

  $plugins['stylescombo'] = array(
    'label' => t('Styles dropdown'),
    'internal' => TRUE,
    'buttons' => array(
      'Styles' => array(
        'label' => t('Font style'),
        'image_alternative' => '<span class="ckeditor-button-dropdown">' . t('Styles') . '<span class="ckeditor-button-arrow"></span></span>',
      ),
    ),
    'config callback' => 'editor_ckeditor_stylescombo_config',
    'settings form' => 'editor_ckeditor_ckeditor_plugin_stylescombo_settings_form',
  );

  $plugins['language'] = array(
    'label' => t('Language'),
    'internal' => TRUE,
    'buttons' => array(
      'Language' => array(
        'label' => t('Language'),
        'required_html' => array(
          array(
            'tags' => array('a', 'span'),
            'attributes' => array('href', 'hreflang', 'lang', 'dir'),
          ),
        ),
        'image' => $image_prefix . '/language.png',
      ),
    ),
    'libraries' => array(array('editor_ckeditor', 'editor_ckeditor.plugins.language')),
    'css' => array(drupal_get_path('module', 'editor_ckeditor') . '/css/plugins/language/editor_ckeditor.language.css'),
    'config callback' => 'editor_ckeditor_language_config',
    'settings form' => 'editor_ckeditor_ckeditor_plugin_language_settings_form',
  );

  // The drupalimage plugin replaces normal image functionality.
  $plugins['drupalimage'] = array(
    'label' => t('Image'),
    'file' => drupal_get_path('module', 'editor_ckeditor') . '/js/plugins/drupalimage/plugin.js',
    'buttons' => array(
      'DrupalImage' => array(
        'label' => t('Image'),
        'required_html' => array(
          array(
            'tags' => array('img'),
            'attributes' => array('src', 'alt', 'data-entity-type', 'data-entity-id', 'data-align'),
          ),
        ),
        'image' => drupal_get_path('module', 'editor_ckeditor') . '/js/plugins/drupalimage/icons/drupalimage.png',
      ),
    ),
    'libraries' => array(array('system', 'drupal.ajax')),
    'config callback' => 'editor_ckeditor_drupalimage_config',
    'settings form' => 'editor_ckeditor_ckeditor_plugin_drupalimage_settings_form',
  );

  // The drupalcaption plugin provides consistent behaviors for image captions.
  $plugins['drupalimagecaption'] = array(
    'file' => drupal_get_path('module', 'editor_ckeditor') . '/js/plugins/drupalimagecaption/plugin.js',
    'contextual' => TRUE,
    'internal' => FALSE,
    'enabled callback' => 'editor_ckeditor_image_plugin_check',
    'required_html' => array(
      array(
        'tags' => array('img'),
        'attributes' => array('data-caption'),
      ),
      array(
        'tags' => array('figure', 'figcaption'),
        'attributes' => array('src', 'alt', 'class', 'data-placeholder'),
      ),
    ),
    'libraries' => array(array('editor_ckeditor', 'editor_ckeditor.plugins.drupalimagecaption')),
    'css' => array(drupal_get_path('module', 'editor_ckeditor') . '/css/plugins/drupalimagecaption/editor_ckeditor.drupalimagecaption.css'),
    'config callback' => 'editor_ckeditor_drupalimagecaption_config',
  );

  // The drupallink plugin replaces normal link functionality.
  $plugins['drupallink'] = array(
    'file' => drupal_get_path('module', 'editor_ckeditor') . '/js/plugins/drupallink/plugin.js',
    'buttons' => array(
      'DrupalLink' => array(
        'label' => t('Link'),
        'required_html' => array(array(
          'tags' => array('a'),
          'attributes' => array('href'),
        )),
        'image' => drupal_get_path('module', 'editor_ckeditor') . '/js/plugins/drupallink/icons/drupallink.png',
      ),
      'DrupalUnlink' => array(
        'label' => t('Link'),
        'required_html' => array(array(
          'tags' => array('a'),
          'attributes' => array('href'),
        )),
        'image' => drupal_get_path('module', 'editor_ckeditor') . '/js/plugins/drupallink/icons/drupalunlink.png',
      ),
    ),
    'libraries' => array(array('system', 'drupal.ajax')),
    'config callback' => 'editor_ckeditor_drupallink_config',
  );

  return $plugins;
}

/**
 * Implements hook_editor_ckeditor_css_alter().
 */
function editor_ckeditor_editor_ckeditor_css_alter(array &$css, $format) {
  // Add the editor caption CSS if the text format associated with this text
  // editor uses the editor_caption filter. This is used by the included
  // CKEditor DrupalImageCaption plugin.
  if (isset($format->filters['editor_caption']) && $format->filters['editor_caption']->status) {
    $css[] = drupal_get_path('module', 'editor') . '/css/filter/filter.caption.css';
  }
}

/**
 * Config for the "internal" CKEditor plugin.
 */
function editor_ckeditor_internal_config($format) {
  // Reasonable defaults that provide expected basic behavior.
  $config = array(
    // Don't load CKEditor's config.js file.
    'customConfig' => '',
    'pasteFromWordPromptCleanup' => TRUE,
    'resize_dir' => 'vertical',
    'justifyClasses' => array('text-align-left', 'text-align-center', 'text-align-right', 'text-align-justify'),
    'entities' => FALSE,
    'disableNativeSpellChecker' => FALSE,
    'indentClasses' => array('indent1', 'indent2', 'indent3'),
    'coreStyles_strike' => array('element' => 'del'),
    'coreStyles_underline' => array('element' => 'span', 'attributes' => array('class' => 'underline')),
  );

  // Add the allowedContent setting, which ensures CKEditor only allows tags
  // and attributes that are allowed by the text format for this text editor.
  list($config['allowedContent'], $config['disallowedContent']) = editor_ckeditor_get_acf_settings($format);

  // Add the format_tags setting, if its button is enabled.
  foreach ($format->editor_settings['toolbar'] as $row) {
    foreach ($row as $button_group) {
      if (in_array('Format', $button_group['items'])) {
        $config['format_tags'] = editor_ckeditor_generate_format_tags_setting($format);
      }
    }
  }

  return $config;
}

/**
 * Config for the "stylescombo" CKEditor plugin.
 */
function editor_ckeditor_stylescombo_config($format) {
  $config = array();
  $settings = $format->editor_settings;

  if (!isset($settings['plugins']['stylescombo']['styles'])) {
    return $config;
  }

  $styles = $settings['plugins']['stylescombo']['styles'];
  $config['stylesSet'] = editor_ckeditor_generate_styles_set_setting($styles);

  return $config;
}

/**
 * Config for the "language" CKEditor plugin.
 */
function editor_ckeditor_language_config($format) {
  // Add the language list if configured.
  $language_list = array();
  $config = array('language_list' => 'un');

  if (isset($format->editor_settings['plugins']['language'])) {
    $config = $format->editor_settings['plugins']['language'];
  }

  $predefined_languages = ($config['language_list'] === 'all') ?
    editor_ckeditor_standard_language_list() :
    editor_ckeditor_united_nations_language_list();

  // Generate the language_list setting as expected by the CKEditor Language
  // plugin, but key the values by the full language name so that we can sort
  // them later on.
  foreach ($predefined_languages as $langcode => $lang) {
    $english_name = $lang[0];
    $direction = empty($lang[2]) ? NULL : $lang[2];

    if ($direction === 'rtl') {
      $language_list[$english_name] = $langcode . ':' . $english_name . ':rtl';
    }
    else {
      $language_list[$english_name] = $langcode . ':' . $english_name;
    }
  }

  // Sort on full language name.
  ksort($language_list);
  $config['language_list'] = array_values($language_list);

  return $config;
}

/**
 * Config for the "drupalimage" CKEditor plugin.
 */
function editor_ckeditor_drupalimage_config($format) {
  // Create a token for access to dialogs.
  $image_token = editor_dialog_token($format, 'image');

  return array(
    'drupalImage_dialogTitleAdd' => t('Insert Image'),
    'drupalImage_dialogTitleEdit' => t('Edit Image'),
    'drupalImage_dialogUrl' => url('editor-ckeditor/dialog/image/'. $format->format, array('query' => array('_format' => 'drupal_dialog', 'token' => $image_token, 'calling_path' => $_GET['q']))),
  );
}

/**
 * Config for the "drupalimagecaption" CKEditor plugin.
 */
function editor_ckeditor_drupalimagecaption_config($format) {
  return array(
    'image2_captionedClass' => 'caption caption-img',
    'image2_alignClasses' => array('align-left', 'align-center', 'align-right'),
    'drupalImageCaption_captionPlaceholderText' => t('Enter caption text here.'),
    // Only enable those parts of DrupalImageCaption for which the
    // corresponding Drupal text filters are enabled.
    'drupalImageCaption_captionFilterEnabled' => !empty($format->filters['editor_caption']->status),
    'drupalImageCaption_alignFilterEnabled' => !empty($format->filters['editor_align']->status),
  );
}

/**
 * Config for the "drupallink" CKEditor plugin.
 */
function editor_ckeditor_drupallink_config($format) {
  // Create a token for access to dialogs.
  $link_token = editor_dialog_token($format, 'link');

  return array(
    'drupalLink_dialogTitleAdd' => t('Add Link'),
    'drupalLink_dialogTitleEdit' => t('Edit Link'),
    'drupalLink_dialogUrl' => url('editor-ckeditor/dialog/link/' . $format->format, array('query' => array('_format' => 'drupal_dialog', 'token' => $link_token, 'calling_path' => $_GET['q']))),
  );
}

/**
 * Settings form for the "stylescombo" CKEditor plugin.
 */
function editor_ckeditor_ckeditor_plugin_stylescombo_settings_form ($form, $form_state, $format) {
  // Defaults.
  $config = array('styles' => '');
  $settings = $format->editor_settings;
  if (isset($settings['plugins']['stylescombo'])) {
    $config = $settings['plugins']['stylescombo'];
  }

  $form['styles'] = array(
    '#title' => t('Styles'),
    '#title_display' => 'invisible',
    '#type' => 'textarea',
    '#default_value' => $config['styles'],
    '#description' => t('A list of classes that will be provided in the "Styles" dropdown. Enter one or more classes on each line in the format: element.classA.classB|Label. Example: h1.title|Title. Advanced example: h1.fancy.title|Fancy title.<br />These styles should be available in your theme\'s CSS file.'),
    '#attached' => array(
      'library' => array(array('editor_ckeditor', 'editor_ckeditor.stylescombo.admin')),
    ),
    '#element_validate' => array('editor_ckeditor_allowed_styles_value'),
  );

  return $form;
}

/**
 * Settings form for the "drupalimage" CKEditor plugin.
 */
function editor_ckeditor_ckeditor_plugin_drupalimage_settings_form ($form, $form_state, $format) {
  $form += editor_ckeditor_image_upload_settings_form($format);
  $form['#attached']['library'][] = array('editor_ckeditor', 'editor_ckeditor.drupalimage.admin');
  return $form;
}

/**
 * Settings form for the "language" CKEditor plugin.
 */
function editor_ckeditor_ckeditor_plugin_language_settings_form ($form, $form_state, $format) {
  return editor_ckeditor_language_settings_form($format);
}

/**
 * Subform constructor to configure the text editor's image upload settings.
 *
 * Each text editor plugin that is configured to offer the ability to insert
 * images and uses EditorImageDialog for that, should use this form to update
 * the text editor's configuration so that EditorImageDialog knows whether it
 * should allow the user to upload images.
 *
 * @param object $format
 *   The text format that is being edited.
 *
 * @return array
 *   The image upload settings form.
 *
 * @ingroup forms
 */
function editor_ckeditor_image_upload_settings_form($format) {
  // Defaults.
  $settings = isset($format->editor_settings['plugins']['drupalimage']) ? $format->editor_settings['plugins']['drupalimage'] : array();

  $settings += array(
    'status' => FALSE,
    'scheme' => file_default_scheme(),
    'directory' => 'inline-images',
    'max_size' => '',
    'max_dimensions' => array('width' => '', 'height' => ''),
  );

  $form['status'] = array(
    '#type' => 'checkbox',
    '#title' => t('Enable image uploads'),
    '#default_value' => $settings['status'],
    '#attributes' => array(
      'data-editor-image-upload' => 'status',
    ),
  );

  $show_if_image_uploads_enabled = array(
    'visible' => array(
      ':input[data-editor-image-upload="status"]' => array('checked' => TRUE),
    ),
  );

  // Any visible, writable wrapper can potentially be used for uploads,
  // including a remote file system that integrates with a CDN.
  $stream_wrappers = file_get_stream_wrappers(STREAM_WRAPPERS_WRITE_VISIBLE);

  foreach ($stream_wrappers as $scheme => $info) {
    $options[$scheme] = $info['description'];
  }

  if (!empty($options)) {
    $form['scheme'] = array(
      '#type' => 'radios',
      '#title' => t('File storage'),
      '#default_value' => $settings['scheme'],
      '#options' => $options,
      '#states' => $show_if_image_uploads_enabled,
      '#access' => count($options) > 1,
    );
  }

  // Set data- attributes with human-readable names for all possible stream
  // wrappers, so that drupal.ckeditor.drupalimage.admin's summary rendering
  // can use that.
  foreach ($stream_wrappers as $scheme => $info) {
    $form['scheme'][$scheme]['#attributes']['data-label'] = t('Storage: @name', array('@name' => $info['name']));
  }

  $form['directory'] = array(
    '#type' => 'textfield',
    '#default_value' => $settings['directory'],
    '#title' => t('Upload directory'),
    '#description' => t("A directory relative to Drupal's files directory where uploaded images will be stored."),
    '#states' => $show_if_image_uploads_enabled,
  );

  $default_max_size = format_size(file_upload_max_size());

  $form['max_size'] = array(
    '#type' => 'textfield',
    '#default_value' => $settings['max_size'],
    '#title' => t('Maximum file size'),
    '#description' => t('If this is left empty, then the file size will be limited by the PHP maximum upload size of @size.', array('@size' => $default_max_size)),
    '#maxlength' => 20,
    '#size' => 10,
    '#placeholder' => $default_max_size,
    '#states' => $show_if_image_uploads_enabled,
  );

  $form['max_dimensions'] = array(
    '#type' => 'item',
    '#title' => t('Maximum dimensions'),
    '#field_prefix' => '<div class="container-inline clearfix">',
    '#field_suffix' => '</div>',
    '#description' => t('Images larger than these dimensions will be scaled down.'),
    '#states' => $show_if_image_uploads_enabled,
  );
  $form['max_dimensions']['width'] = array(
    '#title' => t('Width'),
    '#title_display' => 'invisible',
    '#type' => module_exists('elements') ? 'numberfield' : 'textfield',
    '#default_value' => $settings['max_dimensions']['width'],
    '#size' => 8,
    '#maxlength' => 8,
    '#min' => 1,
    '#max' => 99999,
    '#placeholder' => 'width',
    '#field_suffix' => ' x ',
    '#states' => $show_if_image_uploads_enabled,
  );
  $form['max_dimensions']['height'] = array(
    '#title' => t('Height'),
    '#title_display' => 'invisible',
    '#type' => module_exists('elements') ? 'numberfield' : 'textfield',
    '#default_value' => $settings['max_dimensions']['height'],
    '#size' => 8,
    '#maxlength' => 8,
    '#min' => 1,
    '#max' => 99999,
    '#placeholder' => 'height',
    '#field_suffix' => 'pixels',
    '#states' => $show_if_image_uploads_enabled,
  );

  return $form;
}

/**
 * Subform constructor to configure the text editor's image upload settings.
 *
 * Each text editor plugin that is configured to offer the ability to insert
 * images and uses EditorImageDialog for that, should use this form to update
 * the text editor's configuration so that EditorImageDialog knows whether it
 * should allow the user to upload images.
 *
 * @param object $format
 *   The text format that is being edited.
 *
 * @return array
 *   The image upload settings form.
 *
 * @ingroup forms
 */
function editor_ckeditor_language_settings_form($format) {
  // Defaults.
  $config = array('language_list' => 'un');
  $settings = $format->editor_settings;
  if (isset($settings['plugins']['language'])) {
    $config = $settings['plugins']['language'];
  }

  $predefined_languages = editor_ckeditor_standard_language_list();
  $form['language_list'] = array(
    '#title' => t('Language list'),
    '#title_display' => 'invisible',
    '#type' => 'select',
    '#options' => array(
      'un' => t("United Nations' official languages"),
      'all' => t('All @count languages', array('@count' => count($predefined_languages))),
    ),
    '#default_value' => $config['language_list'],
    '#description' => t('The list of languages to show in the language dropdown. The basic list will only show the <a href="@url">six official languages of the UN</a>. The extended list will show all @count languages that are available in Drupal.', array(
      '@url' => 'http://www.un.org/en/aboutun/languages.shtml/',
      '@count' => count($predefined_languages),
    )),
    '#attached' => array('library' => array(array('editor_ckeditor', 'editor_ckeditor.language.admin'))),
  );

  return $form;
}
