<?php

/**
 * Admin menu callbacks and theme functions for Editor CKEditor module.
 */

/**
 * Editor settings callback; Provide options for Editor CKEditor module.
 */
function editor_ckeditor_settings_form(&$form, $form_state, $format) {
  $plugins = editor_ckeditor_plugins();
  $toolbar_element = theme('editor_ckeditor_settings_toolbar', array('format' => $format, 'plugins' => $plugins));

  $elements['toolbar'] = array(
    '#type' => 'container',
    '#attached' => array(
      'library' => array(
        array('editor_ckeditor', 'editor_ckeditor.admin'),
      ),
      'js' => array(
        array('data' => array('ckeditor' => array('toolbarAdmin' => $toolbar_element)), 'type' => 'setting'),
      ),
    ),
    '#attributes' => array('class' => array('ckeditor-toolbar-configuration')),
  );

  $elements['toolbar']['button_groups'] = array(
    '#type' => 'textarea',
    '#title' => t('Toolbar buttons'),
    '#default_value' => json_encode($format->editor_settings['toolbar']),
    '#attributes' => array('class' => array('ckeditor-toolbar-textarea')),
  );

  // CKEditor plugin settings, if any.
  $elements['plugin_settings_title'] = array(
    '#type' => 'item',
    '#title' => t('CKEditor plugin settings'),
  );
  $elements['plugin_settings'] = array(
    '#type' => 'vertical_tabs',
    '#attributes' => array(
      'id' => 'ckeditor-plugin-settings',
    ),
  );

  $plugins = editor_ckeditor_plugins();

  foreach ($plugins as $plugin_id => $plugin) {
    if (!empty($plugin['settings form'])) {
      $plugin_settings_form = array();
      $function = $plugin['settings form'];

      $elements['plugins'][$plugin_id] = array(
        '#type' => 'fieldset',
        '#title' => $plugin['label'],
        '#open' => TRUE,
        '#group' => 'editor][settings][plugin_settings',
        '#attributes' => array(
          'class' => array('ckeditor-plugin-' . $plugin_id),
        ),
      );

      // Provide enough metadata for the editor_ckeditor.admin library to allow
      // it to automatically show/hide the fieldset containing the settings for
      // this plugin. Only do this if it's a CKEditor plugin that just provides
      // buttons, don't do this if it's a contextually enabled CKEditor plugin.
      // After all, in the latter case, we can't know when its settings should
      // be shown!
      if (empty($plugin['contextual']) && !empty($plugin['buttons'])) {
        $elements['plugins'][$plugin_id]['#attributes']['data-ckeditor-feature-dependency'] = implode(' ', array_keys($plugin['buttons']));
      }

      $elements['plugins'][$plugin_id] += $function($plugin_settings_form, $form_state, $format);
    }
  }

  if (count(element_children($elements['plugins'])) === 0) {
    unset($elements['plugins']);
    unset($elements['plugin_settings_title']);
    unset($elements['plugin_settings']);
  }

  array_unshift($form['#validate'], 'editor_ckeditor_settings_form_validate');
  array_unshift($form['#submit'], 'editor_ckeditor_settings_form_submit');

  return $elements;
}

/**
 * Additional validate handler for filter_admin_format_form().
 */
function editor_ckeditor_settings_form_validate($form, &$form_state) {
}

/**
 * Additional submit handler for filter_admin_format_form().
 */
function editor_ckeditor_settings_form_submit($form, &$form_state) {
  form_set_value($form['editor']['settings']['subform']['toolbar'], json_decode($form_state['values']['editor']['settings']['toolbar']['button_groups'], TRUE), $form_state);
  unset($form_state['values']['editor']['settings']['toolbar']['button_groups']);

  // Remove the plugin settings' vertical tabs state; no need to save that.
  if (!empty($form_state['values']['editor']['settings']['plugins'])) {
    unset($form_state['values']['editor']['settings']['plugin_settings']);
  }
}
